const express = require('express');
const { defaultServerEvents } = require('../utils/defaults');
const ClodeRouter = require('../router');

/**
 * Server is a main Clode instance, that bootstraps Express instance.
 * @constructor You can pass object as an argument with router, config, and port properties to start server immediately. Router must be Clode's Router instance or Express router instance. Config must be an array of middlewares you want to use, and port is a number (default is 3000).
 */
class Server {
    constructor(props) {
        this.app = express();
        this.events = defaultServerEvents;
        this._bootstrap(props);
        this.db = null;
        this.models = [];
    }

    /**
     * Method to bootstrap server if constructor props are passed.
     * @param {object} props
     * @private
     */
    _bootstrap(props) {
        if (props) {
            if (props.router) this.router(props.router);
            if (props.config) {
                props.config.forEach(item => {
                    this.app.use(item);
                });
            }
            this.listen(props.port || 3000);
        }
    }

    /**
     * Starts a listener on defined port
     * @param {number} port Port on what to listent.
     * @default port 3000
     */
    listen(port = 3000) {
        if (this.listener) this.listener.close();
        if (this.db) this.db.connect().then(this.events['db:open'].bind(this));
        this.listener = this.app.listen(
            port,
            this.events.open.bind(this, port)
        );

        return this;
    }

    /**
     * Closes listener
     */
    close() {
        if (!this.listener) throw new Error('No server started');
        if (this.db)
            this.db.disconnect().then(this.events['db:close'].bind(this));
        this.listener.close(this.events.close.bind(this));

        return this;
    }

    /**
     * Sets event listener for server events.
     * @param {string} eventType Could be `open` or `close`.
     * @param {function} handler Function to handle event.
     */
    setEventListener(eventType, handler) {
        if (!eventType) throw new Error('No event type provided');
        if (!handler) throw new Error('No handler provided');
        this.events[eventType] = handler;

        return this;
    }

    /**
     * Function to setup server parameters.
     * @param {function} configurator Function which recieves current Clode.Server instance's `this` and current Express instance as an argument.
     */
    configure(configurator) {
        configurator.call(this, this.app);

        return this;
    }

    /**
     * Setting up router for current server instance.
     * @param {any} routerInstance Could be Clode.Router or express.Router as well.
     */
    router(routerInstance) {
        if (!routerInstance) throw new Error('No router passed');

        if (routerInstance.name) {
            this.app.use(routerInstance);
        }

        if (routerInstance instanceof ClodeRouter) {
            this.app.use(routerInstance.router);
        }

        return this;
    }

    database(mongoritoInstance) {
        this.db = mongoritoInstance;
    }

    model(modelInstances) {
        if (!this.db) throw new Error('No database reference exists');
        if (Array.isArray(modelInstances)) {
            this.models = this.models.concat(modelInstances);
            for (let index = 0; index < modelInstances.length; index++) {
                this.db.register(modelInstances[index]);
            }
        } else {
            this.models.push(modelInstances);
            this.db.register(modelInstances);
        }
    }
}

module.exports = Server;
