const chai = require('chai');
const chaiHttp = require('chai-http');
const Server = require('../server');
const Router = require('../router');

const { expect } = chai;
const should = chai.should();

const AppRouter = new Router();
AppRouter.get('/test', (req, res) => res.json([]));

chai.use(chaiHttp);

describe('Server', function() {
    it('Should create instance if constructor params are passed and create route', function(done) {
        const clode = new Server({
            router: AppRouter,
            port: 3000
        });

        chai.request(clode.app)
            .get('/test')
            .end((err, res) => {
                res.should.have.status(200);
                res.body.should.be.a('array');
                res.body.length.should.be.eql(0);
                clode.close();
                done();
            });
    });

    it('Should apply middleware on fast bootstrap', function(done) {
        const clode = new Server({
            router: AppRouter,
            config: [() => {}],
            port: 3000
        });

        clode.close();

        done();
    });

    it('Should create instance on .listen() call', function(done) {
        const clode = new Server();
        clode.router(AppRouter);
        clode.listen(3001);

        chai.request(clode.app)
            .get('/test')
            .end((err, res) => {
                res.should.have.status(200);
                res.body.should.be.a('array');
                res.body.length.should.be.eql(0);
                clode.close();
                done();
            });
    });

    it('Should dynamically change port', function(done) {
        const clode = new Server();
        clode.router(AppRouter);
        clode.listen(8000);

        chai.request('http://localhost:8000')
            .get('/test')
            .end((err, res) => {
                res.should.have.status(200);
                res.body.should.be.a('array');
                res.body.length.should.be.eql(0);

                clode.listen(8080);

                chai.request('http://localhost:8080')
                    .get('/test')
                    .end((err, res) => {
                        res.should.have.status(200);
                        res.body.should.be.a('array');
                        res.body.length.should.be.eql(0);

                        clode.close();
                        done();
                    });
            });
    });

    it('Should set up event listener for open & close', function(done) {
        const clode = new Server();
        clode.router(AppRouter);
        clode
            .setEventListener('open', () => {
                clode.close();
            })
            .setEventListener('close', () => {
                done();
            })
            .listen(8081);
    });

    it('Should throw error when event name not passed', function(done) {
        const clode = new Server();
        clode.router(AppRouter);
        expect(function() {
            clode.setEventListener();
        }).to.throw('No event type provided');
        done();
    });

    it('Should call configurator function with app argument', function(done) {
        const clode = new Server();
        clode.configure(app => {
            expect(typeof app === 'function').to.be.true;
            done();
        });
    });

    it('Should throw error on close if server is not started', function(done) {
        const clode = new Server();
        expect(() => {
            clode.close();
        }).to.throw('No server started');
        done();
    });
});
